/*
 * Copyright 2010-2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License").
 * You may not use this file except in compliance with the License.
 * A copy of the License is located at
 *
 *  http://aws.amazon.com/apache2.0
 *
 * or in the "license" file accompanying this file. This file is distributed
 * on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package software.amazon.awssdk.enhanced.dynamodb.converters.attribute;

import static org.assertj.core.api.Assertions.assertThatThrownBy;

import org.assertj.core.api.ThrowableAssert;
import software.amazon.awssdk.enhanced.dynamodb.AttributeConverter;
import software.amazon.awssdk.enhanced.dynamodb.internal.converter.attribute.EnhancedAttributeValue;
import software.amazon.awssdk.services.dynamodb.model.AttributeValue;


public final class ConverterTestUtils {
    private ConverterTestUtils() {}

    public static <T> AttributeValue transformFrom(AttributeConverter<T> converter, T value) {
        return converter.transformFrom(value);
    }

    public static <T> T transformTo(AttributeConverter<T> converter, AttributeValue value) {
        return converter.transformTo(value);
    }

    public static <T> T transformTo(AttributeConverter<T> converter, EnhancedAttributeValue value) {
        return converter.transformTo(value.toAttributeValue());
    }

    public static void assertFails(ThrowableAssert.ThrowingCallable shouldRaiseThrowable) {
        assertThatThrownBy(shouldRaiseThrowable).isInstanceOf(IllegalArgumentException.class);
    }
}
